<?php
/**
 * Droppics
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droppics
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

use Imagine\Gd\Imagine;
use Imagine\Image\Point;
use Imagine\Image\Box;

/**
 *  Class DroppicsImagine
 */
class DroppicsImagine
{

    /**
     * Resize to string
     *
     * @param string $file          File
     * @param float  $width         Width
     * @param float  $height        Height
     * @param string $imageMimeType Mine type
     * @param string $fit           Fit
     *
     * @return mixed
     */
    public static function resizeToString($file, $width, $height, $imageMimeType = 'image/jpeg', $fit = 'inside')
    {
        $imagine = new Imagine();
        $box = new Box($width, $height);
        $image = $imagine->open($file);

        //we scale the image to make the smaller dimension fit our resize box
        $filter = $fit === 'inside' ? \Imagine\Image\ImageInterface::THUMBNAIL_INSET : \Imagine\Image\ImageInterface::THUMBNAIL_OUTBOUND;
        $filter =  $filter | \Imagine\Image\ImageInterface::THUMBNAIL_FLAG_NOCLONE;
        $image = $image->thumbnail($box, $filter);

        $imageString = self::getImageString($image, $imageMimeType);

        return $imageString;
    }

    /**
     * Crop
     *
     * @param string $file     File
     * @param float  $x        X position
     * @param float  $y        Y position
     * @param float  $width    Width
     * @param float  $height   Height
     * @param array  $asString As string
     *
     * @return \Imagine\Gd\Image|\Imagine\Image\ImageInterface|\Imagine\Image\ManipulatorInterface|mixed
     */
    public static function crop($file, $x, $y, $width, $height, $asString = array())
    {
        $imagine = new Imagine();
        $box = new Box($width, $height);
        $image = $imagine->open($file);
        //original size
        $srcBox = $image->getSize();
        //we scale on the smaller dimension
        if ($srcBox->getWidth() > $box->getWidth()) {
            switch ($x) {
                case 'left':
                    $posX = 0;
                    break;
                case 'center':
                    $posX = ($srcBox->getWidth() - $box->getWidth()) / 2;
                    break;
                case 'right':
                    $posX = $srcBox->getWidth() - $box->getWidth();
            }
        } else {
            $posX = 0;
            $width = $srcBox->getWidth();
        }

        if ($srcBox->getHeight() > $box->getHeight()) {
            switch ($y) {
                case 'top':
                    $posY = 0;
                    break;
                case 'middle':
                    $posY = ($srcBox->getHeight() - $box->getHeight()) / 2;
                    break;
                case 'bottom':
                    $posY = $srcBox->getHeight() - $box->getHeight();
            }
        } else {
            $posY = 0;
            $height = $srcBox->getHeight();
        }

        $cropPoint = new Point($posX, $posY);

        $newBox = new Box($width, $height);
        //we scale the image to make the smaller dimension fit our resize box
        //$filter = $fit === 'inside' ? \Imagine\Image\ImageInterface::THUMBNAIL_INSET : \Imagine\Image\ImageInterface::THUMBNAIL_OUTBOUND;
        $image = $image->crop($cropPoint, $newBox);
        $result = $image;
        if (!empty($asString)) {
            $imageString = self::getImageString($image, $asString['pic_format']);
            $result = $imageString;
        }

        return $result;
    }

    /**
     * Get Image String
     *
     * @param Image  $image         Image
     * @param string $imageMimeType Mine type
     *
     * @return mixed
     */
    public static function getImageString($image, $imageMimeType = 'png')
    {
        if ($imageMimeType === 'image/jpeg') {
            $imageString = $image->get('jpg', array('quantity' => droppicsBase::getParam('jpg_quality', 80)));
        } elseif ($imageMimeType === 'image/png') {
            $imageString = $image->get('png', array('quantity' => droppicsBase::getParam('png_quality', 9)));
        } elseif ($imageMimeType === 'image/gif') {
            $imageString = $image->get('gif');
        } elseif ($imageMimeType === 'image/webp') {
            $imageString = $image->get('webp');
        }
        return $imageString;
    }
}
