<?php
/**
 * Droppics
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droppics
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

jimport('joomla.filesystem.folder');

/**
 * Class DroppicsControllerCategories
 */
class DroppicsControllerCategories extends JControllerAdmin
{


    /**
     * Proxy for getModel
     *
     * @param string $name   The model name. Optional.
     * @param string $prefix The class prefix. Optional.
     * @param array  $config The class config. Optional.
     *
     * @return object    The model.
     * @since  1.6
     */
    public function getModel($name = 'Category', $prefix = 'DroppicsModel', $config = array('ignore_request' => true))
    {
        $model = parent::getModel($name, $prefix, $config);
        return $model;
    }

    /**
     * Removes an item.
     *
     * @return void
     *
     * @since 11.1
     */
    public function delete()
    {
        // Check for request forgeries
        JSession::checkToken() || die(JText::_('JINVALID_TOKEN'));

        // Get items to remove from the request.
        $cid = JFactory::getApplication()->input->getInt('id_gallery', 0);

        if ($cid) {
            // Get the model.
            $model = $this->getModel();
            JFactory::getApplication()->setUserState('list.limit', 100000);

            $modelC = $this->getModel('categories');
            $modelC->setState('category.id', $cid);
            $modelC->setState('filter.parentId', $cid);
            $modelC->setState('category.recursive', true);
            $items = $modelC->getItems();

            $canDo = DroppicsHelper::getActions();
            if ($canDo->get('core.delete')) {
                if (!$canDo->get('core.edit')) {
                    if ($canDo->get('core.edit.own')) {
                        $gallery = $model->getItem($cid);
                        if ($gallery->created_user_id !== JFactory::getUser()->id) {
                            $this->exitStatus('not permitted');
                        }
                    } else {
                        $this->exitStatus('not permitted');
                    }
                }
            } else {
                $this->exitStatus('not permitted');
            }

            //little hack because joomla always delete children, bug in version 3.1
            $joomla31 = false;
            if (version_compare(droppicsBase::getJoomlaVersion(), '3.1')) {
                $joomla31 = true;
            }


            $errors = array();
            // Remove the items.
            if ($model->delete($cid)) {
                //todo : delete files from database
                $path = droppicsBase::getPicturePath($cid[0]);
                if (is_dir($path)) {
                    if (!JFolder::delete($path)) {
                        $errors[] = 'error while deleting directory, please delete folder ' . $path . ' manually'; //todo: translate
                    }
                }
                //delete children
                foreach ($items as $item) {
                    if ($item->id === $cid[0]) {
                        continue;
                    }
                    $id = $item->id;
                    if ($model->delete($id) || $joomla31) {
                        $path = droppicsBase::getPicturePath($item->id);
                        if (is_dir($path)) {
                            if (!JFolder::delete($path)) {
                                $errors[] = 'error while deleting directory, please delete folder ' . $path . ' manually'; //todo: translate
                            }
                        }
                    } else {
                        $errors[] = 'error while deleting gallery'; //todo: translate
                    }
                }
            } else {
                $errors[] = $model->getError();
            }
            if (count($errors)) {
                $this->exitStatus(implode('<br/>', $errors));
            }
            $this->exitStatus(true);
        }
    }

    /**
     * Order
     *
     * @return void
     */
    public function order()
    {
        $position = JFactory::getApplication()->input->getCmd('position', 'after');
        $pk = JFactory::getApplication()->input->getInt('pk', null);
        $ref = JFactory::getApplication()->input->getInt('ref');
        $id_gallery = JFactory::getApplication()->input->getCmd('id_gallery', 0);
        $model = $this->getModel();
        $canDo = DroppicsHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $gallery = $model->getItem($pk);
                if ($gallery->created_user_id !== JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            }
        }
        if ($ref === 0) {
            $ref = 1;
        }
        if ($position !== 'after') {
            $position = 'first-child';
        }

        $table = $model->getTable();
        if ($table->moveByReference($ref, $position, $pk)) {
            $listoption = $this->getlistCategory($id_gallery);
            $this->exitStatus(true, array('position' => $pk . ' ' . $position . ' ' . $ref, 'listoptions' => $listoption));
        }
        $this->exitStatus('problem');
    }

    /**
     * Get lists categories
     *
     * @param integer $id_gallery ID gallery
     *
     * @return string
     */
    public function getlistCategory($id_gallery)
    {
        $model = $this->getModel('categories');
        $model->setState('filter.extension', 'com_droppics');
        $categories = $model->getItems();
        $content = '';
        $content .= '<option value ="0">' . JText::_('COM_DROPPICS_VIEW_ALL_CATEGORY') . '</option >';

        if (!empty($categories)) {
            foreach ($categories as $index => $category) {
                $pad = 2 * ($category->level - 1);
                $style = str_repeat('&nbsp;', ($pad >= 0) ? $pad : 0);
                $selected = 'selected ="selected"';
                if ($category->id === $id_gallery) {
                    $content .= '<option ' . $selected . ' value = ' .
                        $category->id . '>' . $style . $category->title . '</option >';
                } else {
                    $content .= '<option value = ' . $category->id . '> ' . $style . $category->title . '</option>';
                }
            }
        }
        return $content;
    }

    /**
     * Return a json response
     *
     * @param boolean $status Status
     * @param array   $datas  Array of datas to return with the json string
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
}
