<?php
/**
 * Droppics
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droppics
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

/**
 * Class Droppics Controller Category
 */
class DroppicsControllerCategory extends JControllerForm
{
    /**
     * Saved category id
     *
     * @var string
     */
    private $savedId;


    /**
     * Set a file title
     *
     * @return void
     * @throws \Exception Throw when application can not start
     * @since  1.0
     */
    public function setTitle()
    {
        $id_category = JFactory::getApplication()->input->getInt('id_gallery', 0);

        $model = $this->getModel();
        $canDo = DroppicsHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $gallery = $model->getItem($id_category);
                if ($gallery->created_user_id !== JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            } else {
                $this->exitStatus('not permitted');
            }
        }


        $title = JFactory::getApplication()->input->getString('title');

        $model = $this->getModel();
        if ($model->setTitle($id_category, $title)) {
            $return = true;
        } else {
            $return = false;
        }
        echo json_encode($return);
        JFactory::getApplication()->close();
    }

    /**
     * Method to add a category
     *
     * @return void
     */
    public function addCategory()
    {
        $canDo = DroppicsHelper::getActions();
        if (!$canDo->get('core.create')) {
            $this->exitStatus('not permitted');
        }

        $datas = array();
        $datas['jform']['extension'] = 'com_droppics';
        $datas['jform']['title'] = JText::_('COM_DROPPICS_MODEL_GALLERY_DEFAULT_NAME');
        $datas['jform']['alias'] = JText::_('COM_DROPPICS_MODEL_GALLERY_DEFAULT_NAME') . '-' . date('dmY-h-m-s', time());
        $datas['jform']['parent_id'] = 1;
        // $datas['jform']['published'] = 1;
        $datas['jform']['language'] = '*';
        $datas['jform']['metadata']['tags'] = '';

        //Set state value to retreive the correct table
        $model = $this->getModel();
        $model->setState('category.extension', 'category');

        $app = JFactory::getApplication();
        $app->input->set('id', 0);
        foreach ($datas as $data => $val) {
            JFactory::getApplication()->input->set($data, $val, 'POST');
        }
        if ($this->save()) {
            $this->exitStatus(true, array('id_category' => $this->savedId, 'name' => JText::_('COM_DROPPICS_MODEL_GALLERY_DEFAULT_NAME')));
        }
        $this->exitStatus('error while adding gallery');
    }

    /**
     * Method run after save J3
     *
     * @param JModelLegacy $model     Model instance
     * @param array        $validData Data
     *
     * @return void
     */
    protected function postSaveHookJ3(JModelLegacy $model, $validData = array())
    {
        $this->savedId = $model->getState($model->getName() . '.id');
        parent::postSaveHook($model, $validData);
    }

    /**
     * Save params config
     *
     * @return void
     */
    public function setParams()
    {
        $datas = JFactory::getApplication()->input->getVar('jform', null, 'default', 'array');

        $model = $this->getModel();
        $canDo = DroppicsHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $gallery = $model->getItem((int)$datas['id']);
                if ($gallery->created_user_id !== JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            } else {
                $this->exitStatus('not permitted');
            }
        }

        $item = get_object_vars($model->getItem((int)$datas['id']));
        $item['access'] = (int)$datas['access'];


        //Set state value to retreive the correct table
        $model->setState('category.extension', 'categoryparams');
        $this->input->post->set('jform', $item);
        $id = parent::save();
        if ($id) {
            $this->exitStatus(true);
        }
        $this->exitStatus('error while saving params');
    }


    /**
     * Return a json response
     *
     * @param boolean $status Response status
     * @param array   $datas  Array of datas to return with the json string
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }


    /**
     * Get redirect to item append
     *
     * @param null   $recordId Record id
     * @param string $urlVar   Url var
     *
     * @return string
     */
    public function getRedirectToItemAppend($recordId = null, $urlVar = 'id')
    {
        $append = parent::getRedirectToItemAppend($recordId, $urlVar);

        $format = JFactory::getApplication()->input->getCmd('format', 'raw');

        // Setup redirect info.
        if ($format) {
            $append .= '&format=' . $format;
        }
        return $append;
    }

    /**
     * Check permission to edit
     *
     * @param array  $data Data
     * @param string $key  Key
     *
     * @return boolean
     * @throws \Exception Throw when application can not start
     * @since  version
     */
    protected function allowEdit($data = array(), $key = 'id')
    {
        $canDo = DroppicsHelper::getActions();
        if ($canDo->get('core.edit') || $canDo->get('core.edit.own')) {
            return true;
        }
    }

    /**
     * Check allow add
     *
     * @param array $data Data
     *
     * @return boolean
     * @since  1.0
     */
    protected function allowAdd($data = array())
    {
        $canDo = DroppicsHelper::getActions();
        if ((int)($canDo->get('core.create'))) {
            return true;
        }
        return false;
    }

    /**
     * Method to save a record.
     *
     * @param string $key    The name of the primary key of the URL variable.
     * @param string $urlVar The name of the URL variable if different from the primary key (sometimes required to avoid router collisions).
     *
     * @return boolean  True if successful, false otherwise.
     *
     * @since 12.2
     */
    public function save($key = null, $urlVar = null)
    {
        // Check for request forgeries.
        JSession::checkToken() || jexit(JText::_('JINVALID_TOKEN'));

        $app = JFactory::getApplication();
        $lang = JFactory::getLanguage();
        $model = $this->getModel();
        $table = $model->getTable();
        $data = $app->input->get('jform', array(), 'array');
        $checkin = property_exists($table, 'checked_out');
        $context = $this->option . '.edit.' . $this->context;
        $task = $this->getTask();

        // Determine the name of the primary key for the data.
        if (empty($key)) {
            $key = $table->getKeyName();
        }

        // To avoid data collisions the urlVar may be different from the primary key.
        if (empty($urlVar)) {
            $urlVar = $key;
        }

        $recordId = $app->input->getInt($urlVar);

        // Populate the row id from the session.
        $data[$key] = $recordId;

        // Access check.
        if (!$this->allowSave($data, $key)) {
            $app->enqueueMessage(JText::_('JLIB_APPLICATION_ERROR_SAVE_NOT_PERMITTED'), 'error');

            $this->setRedirect(
                JRoute::_(
                    'index.php?option=' . $this->option . '&view=' . $this->view_list
                    . $this->getRedirectToListAppend(),
                    false
                )
            );

            return false;
        }

        // Validate the posted data.
        // Sometimes the form needs some posted data, such as for plugins and modules.
        $form = $model->getForm($data, false);

        if (!$form) {
            $app->enqueueMessage($model->getError(), 'error');

            return false;
        }

        // Test whether the data is valid.
        $validData = $model->validate($form, $data);

        // Check for validation errors.
        if ($validData === false) {
            // Get the validation messages.
            $errors = $model->getErrors();

            // Push up to three validation messages out to the user.
            for ($i = 0, $n = count($errors); $i < $n && $i < 3; $i++) {
                if ($errors[$i] instanceof Exception) {
                    $app->enqueueMessage($errors[$i]->getMessage(), 'warning');
                } else {
                    $app->enqueueMessage($errors[$i], 'warning');
                }
            }

            // Save the data in the session.
            $app->setUserState($context . '.data', $data);

            // Redirect back to the edit screen.
            $this->setRedirect(
                JRoute::_(
                    'index.php?option=' . $this->option . '&view=' . $this->view_item
                    . $this->getRedirectToItemAppend($recordId, $urlVar),
                    false
                )
            );

            return false;
        }

        // Attempt to save the data.
        $validData['published'] = 1;
        if (!$model->save($validData)) {
            // Save the data in the session.
            $app->setUserState($context . '.data', $validData);

            // Redirect back to the edit screen.
            $app->enqueueMessage(JText::sprintf('JLIB_APPLICATION_ERROR_SAVE_FAILED', $model->getError()), 'error');

            $this->setRedirect(
                JRoute::_(
                    'index.php?option=' . $this->option . '&view=' . $this->view_item
                    . $this->getRedirectToItemAppend($recordId, $urlVar),
                    false
                )
            );

            return false;
        }

        $this->setMessage(
            JText::_(
                ($lang->hasKey($this->text_prefix . ($recordId === 0 && $app->isClient('site') ? '_SUBMIT' : '') . '_SAVE_SUCCESS')
                    ? $this->text_prefix
                    : 'JLIB_APPLICATION') . ($recordId === 0 && $app->isClient('site') ? '_SUBMIT' : '') . '_SAVE_SUCCESS'
            )
        );


        // Clear the record id and data from the session.
        $this->releaseEditId($context, $recordId);
        $app->setUserState($context . '.data', null);

        // Redirect to the list screen.
        $this->setRedirect(
            JRoute::_(
                'index.php?option=' . $this->option . '&view=' . $this->view_list
                . $this->getRedirectToListAppend(),
                false
            )
        );

        // Invoke the postSave method to allow for the child class to access the model.
        $this->postSaveHookJ3($model, $validData);
        return true;
    }
}
