<?php
/**
 * Droppics
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droppics
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

jimport('joomla.application.component.controllerform');
jimport('joomla.filesystem.folder');

/**
 * Class Droppics Controller Gallery
 */
class DroppicsControllerGallery extends JControllerForm
{

    /**
     * Method to change a gallery theme
     *
     * @return void
     */
    public function setTheme()
    {
        $id_gallery = JFactory::getApplication()->input->getInt('id_gallery');
        
        $model = $this->getModel('category');
        $canDo = DroppicsHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $gallery = $model->getItem($id_gallery);
                if ($gallery->created_user_id !== JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            } else {
                $this->exitStatus('not permitted');
            }
        }

        
        $theme = JFactory::getApplication()->input->getCmd('theme');
        if ($id_gallery<=0) {
            $this->exitStatus('error');
        }
        $model = $this->getModel();
        
        if (!$model->setTheme($id_gallery, $theme)) {
            $this->exitStatus('error while changing theme');
        }
        $this->exitStatus(true);
    }

    /**
     * Return a json response
     *
     * @param boolean $status Response status
     * @param array   $datas  Array of datas to return with the json string
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
            $response = array('response'=>$status,'datas'=>$datas);
            echo json_encode($response);
            JFactory::getApplication()->close();
    }

    /**
     * Method to save a record.
     *
     * @param string $key    The name of the primary key of the URL variable.
     * @param string $urlVar The name of the URL variable if different from the primary key (sometimes required to avoid router collisions).
     *
     * @return void
     *
     * @since 12.2
     */
    public function save($key = null, $urlVar = null)
    {
        $model = $this->getModel('category');
        $canDo = DroppicsHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $gallery = $model->getItem(JFactory::getApplication()->input->getInt('id', 0));
                if ($gallery->created_user_id !== JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            } else {
                $this->exitStatus('not permitted');
            }
        }
        
        if (parent::save($key, $urlVar)) {
                $this->exitStatus(true);
        }
        $errormsg = $this->getErrors();
        if ($errormsg) {
            $this->exitStatus($errormsg);
        }
        $app = JFactory::getApplication();
        $errors = $app->getMessageQueue();
        if (!empty($errors)) {
            $errormsg='';
            foreach ($errors as $error) {
                $errormsg .= $error['message'].'<br/>';
            }
            $this->exitStatus($errormsg);
        }
        
        $this->exitStatus('error');
    }

    /**
     * Get redirect to item append
     *
     * @param null   $recordId Record id
     * @param string $urlVar   Url var
     *
     * @return string
     */
    public function getRedirectToItemAppend($recordId = null, $urlVar = 'id')
    {
        $append = parent::getRedirectToItemAppend($recordId, $urlVar);
        
        $format = JFactory::getApplication()->input->getCmd('format', 'raw');

        // Setup redirect info.
        if ($format) {
                $append .= '&format=' . $format;
        }
        return $append;
    }

    /**
     * Check permission to edit
     *
     * @param array  $data Data
     * @param string $key  Key
     *
     * @return boolean
     * @throws \Exception Throw when application can not start
     * @since  version
     */
    protected function allowEdit($data = array(), $key = 'id')
    {
        return true;
    }

    /**
     * Check exit gallery
     *
     *  @return void
     */
    public function ajaxCheckExitGallery()
    {
        $model = $this->getModel('gallery');
        $exist = false;
        if ($model->getGalleryTheme(JFactory::getApplication()->input->getInt('id_gallery', 0))) {
            $exist = true;
        } else {
            $exist = false;
        }
        $this->exitStatus(true, array('exist'=> $exist));
        JFactory::getApplication()->close();
    }

    /**
     * Search image
     *
     * @return void
     */
    public function searchimage()
    {
        $model=$this->getModel();
        $datas = $model->getsearchimg($_GET);
        $root_folder = droppicsBase::getParam('change_root_folder_old', 'com_droppics');
        $content = '';
        $content .= '<div id="droppicsgallery'.$_GET['id_gallery'].'" class="droppicsgallery droppicsgallery'.$this->name.'">';
        if (!empty($datas)) {
            foreach ($datas as $data) {
                $pparams = json_decode($data->params);
                // phpcs:disable PHPCompatibility.ParameterValues.NewHTMLEntitiesEncodingDefault
                $customPictures = htmlentities(json_encode($data->custom));
                $content .= '<div class="wimg">';
                $content .= '   <div class="selimg">';
                $content .= '       <img class="img" data-file="'.$data->file.'" data-id-gallery="' .$data->id_gallery.'" data-id-picture="'
                    .$data->id.'" data-picture="'.COM_MEDIA_BASEURL.'/'.$root_folder.'/'.$data->id_gallery.'/'
                    .$data->file.'" data-customs="'.$customPictures.'" src="'.COM_MEDIA_BASEURL.'/'.$root_folder.'/'.$data->id_gallery.'/thumbnails/'.$data->file.'"  alt="'.$data->alt.'" 
                    data-title="' . htmlentities($data->title) . '"/>';
                if (isset($pparams->is_video) && $pparams->is_video === '1') {
                    $content .= '   <i class="video"></i>';
                }
                $content .= '   </div>';
                $content .= '</div>';
            }
        }
        $content .= '<input type="hidden" name="theme" value="'.$this->name.'" />';
        $content .= '</div>';
        echo $content;
        JFactory::getApplication()->close();
    }
}
