<?php
/**
 * Droppics
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droppics
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

jimport('joomla.application.component.helper');

/**
 * Class DroppicsComponentHelper
 */
class DroppicsComponentHelper extends JComponentHelper
{
    /**
     * Method to set config parameters
     *
     * @param array $datas Data
     *
     * @return boolean
     */
    public static function setParams($datas)
    {
        $component = self::getComponent('com_droppics');
        $table = JTable::getInstance('extension');
        // Load the previous Data
        if (!$table->load($component->id, false)) {
            return false;
        }
        $d = json_decode($table->params);
        foreach ($datas as $key => $data) {
            $d->$key = $data;
        }
        $table->params = json_encode($d);
        // Bind the data.
        if (!$table->bind($datas)) {
            return false;
        }
        // Check the data.
        if (!$table->check()) {
            return false;
        }
        // Store the data.
        if (!$table->store()) {
            return false;
        }
        unset(self::$components['com_droppics']);
        return true;
    }

    /**
     * Load theme params
     *
     * @param string $theme         Theme
     * @param null   $global_params Global params
     *
     * @return stdClass
     */
    public static function loadParams($theme = 'default', $global_params = null)
    {
        $ob = new stdClass();
        if ($theme === '') {
            $theme = 'default';
        }
        foreach ($global_params->toArray() as $key => $val) {
            if (strpos($key, $theme . '_') !== false) {
                if ($theme === 'default') {
                    $newKey = str_replace('default_', '', $key);
                    $ob->$newKey = $global_params->get($key);
                } else {
                    $ob->$key = $global_params->get($key);
                }
            }
        }

        return $ob;
    }

    /**
     * Method to get the version of a component
     *
     * @param null|string $option Component name
     *
     * @return null|string
     */
    public static function getVersion($option = 'com_droppics')
    {
        $manifest = self::getManifest($option);
        if ($manifest && property_exists($manifest, 'version')) {
            return $manifest->version;
        }
        return null;
    }

    /**
     * Method to get an object containing the manifest values
     *
     * @param string $option Component name
     *
     * @return object
     */
    public static function getManifest($option)
    {
        $component = self::getComponent($option);
        $table = JTable::getInstance('extension');
        // Load the previous Data
        if (!$table->load($component->id, false)) {
            return false;
        }
        return json_decode($table->manifest_cache);
    }

    /**
     * Filter text for bxSlider
     *
     * @param string $text Text
     *
     * @return string
     */
    public static function bxSliderFilterText($text)
    {
        $text = preg_replace('/<p[^>]*>[\s|&nbsp;|<br>]*<\/p>/', '', $text);
        if ($text === JText::_('PLG_DROPPICS_BXSLIDER_IMAGE_ADD_CONTENT')) {
            return '';
        }
        return JComponentHelper::filterText($text);
    }

    /**
     * Get max upload file size
     *
     * @return integer
     */
    public static function getMaxUploadSize()
    {
        $mediaParams = JComponentHelper::getParams('com_media');
        $maxUpload = (int)$mediaParams->get('upload_maxsize', 10);
        if ($maxUpload <= 0) {
            $max_upload = (int)(ini_get('upload_max_filesize'));
            $max_post = (int)(ini_get('post_max_size'));
            $memory_limit = (int)(ini_get('memory_limit'));
            $maxUpload = min($max_upload, $max_post, $memory_limit);
        }

        return $maxUpload;
    }
}
