<?php
/**
 * Droppics
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droppics
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

defined('_JEXEC') || die;

jimport('joomla.application.component.model');

/**
 * Class DroppicsModelFrontcategories
 */
class DroppicsModelFrontcategories extends JModelLegacy
{

    /**
     * Get categories
     *
     * @param integer $id_gallery ID gallery
     *
     * @return array|boolean
     */
    public function getCategories($id_gallery)
    {
        $db = $this->getDbo();
        $result = array();

        $params_com = JComponentHelper::getParams('com_droppics');
        $t = $params_com->get('lastmodified', '');
        if ($t) {
            $t = '?' . $t;
        }

        $query = $db->getQuery(true);
        $query->select('c.id as id_category');
        $query->select('c.title as category_title');
        $query->from('#__categories as c');
        $query->where('c.parent_id=' . (int)$id_gallery);
        $query->where('c.extension="com_droppics"');
        $query->order('c.lft ASC');
        $db->setQuery($query);
        if (!$db->execute()) {
            return false;
        }
        $cats = $db->loadObjectList();
        if (is_array($cats) && count($cats)) {
            $db->setQuery($query); // Fix loadColumn return empty result
            $catIds = $db->loadColumn();

            $subQuery = 'Select id_gallery, min(position) as minPosition From #__droppics_pictures WHERE id_gallery IN (' . implode(',', $catIds) . ') Group by id_gallery';
            $query = $db->getQuery(true);
            $query->select('p.id as id_picture');
            $query->select('p.file as picture_file');
            $query->select('concat(p.file,"' . $t . '") as picture_ver');
            $query->select('p.title as picture_title');
            $query->select('p.alt as picture_alt');
            $query->select('p.id_gallery as id_gallery');
            $query->from('(' . $subQuery . ') as p1');
            $query->innerJoin('#__droppics_pictures as p ON p.id_gallery = p1.id_gallery AND p.position = p1.minPosition');
            $db->setQuery($query);
            if (!$db->execute()) {
                return false;
            }
            $pictures = $db->loadObjectList('id_gallery');
            foreach ($cats as $cat) {
                if (isset($pictures[$cat->id_category])) {
                    $result[] = (object)array_merge((array)$cat, (array)$pictures[$cat->id_category]);
                }
            }
        }

        return $result;
    }

    /**
     * Get parent category
     *
     * @param integer $id_gallery ID gallery
     *
     * @return boolean
     */
    public function getParent($id_gallery)
    {
        $db = $this->getDbo();

        $params_com = JComponentHelper::getParams('com_droppics');
        $t = $params_com->get('lastmodified', '');
        if ($t) {
            $t = '?' . $t;
        }

        $query = $db->getQuery(true);
        $query->select('c.id as id_category');
        $query->select('c.title as category_title');
        $query->select('p.id as id_picture');
        $query->select('p.file as picture_file');
        $query->select('concat(p.file,"' . $t . '") as picture_ver');
        $query->select('p.title as picture_title');
        $query->select('p.alt as picture_alt');
        $query->from('#__categories as c');
        $query->leftJoin('#__droppics_pictures as p ON p.id_gallery = c.id');
        $query->where('c.id = (SELECT x.parent_id FROM #__categories as x WHERE x.id=' . (int)$id_gallery . ')');
        $query->where('c.extension="com_droppics"');
        $query->order('c.lft ASC');
        $query->group('p.id_gallery');
        $query->order('p.position ASC');

        $db->setQuery($query);
        if (!$db->execute()) {
            return false;
        }
        return $db->loadObject();
    }


    /**
     * Get all id child or parent
     *
     * @param integer $id      ID
     * @param array   $arr     Array
     * @param boolean $getchid Get children
     *
     * @return array|boolean
     */
    public function getAllChildOrParent($id, &$arr, $getchid)
    {
        $db = $this->getDbo();
        $query = $db->getQuery(true);
        if ($getchid) {
            $query->select('id');
            $query->from('#__categories as c');
            $query->where('c.parent_id = ' . (int)$id);
            $db->setQuery($query);
            if ($db->execute()) {
                foreach ($db->loadObjectList() as $idcat) {
                    $arr[] = (int)$idcat->id;
                    $this->getAllChildOrParent($idcat->id, $arr, $getchid);
                }
            } else {
                return false;
            }
        } else {
            $query->select('parent_id');
            $query->from('#__categories as c');
            $query->where('c.id = ' . (int)$id);
            $db->setQuery($query);
            if ($db->execute()) {
                foreach ($db->loadObjectList() as $idcat) {
                    if ((int)$idcat->parent_id > 1) {
                        $arr[] = (int)$idcat->parent_id;
                        $this->getAllChildOrParent($idcat->parent_id, $arr, $getchid);
                    }
                }
            } else {
                return false;
            }
        }
        return $arr;
    }
}
